/*   AM-STD/RGB for ICP-PCI Kernel module 
 *   free public release
 *
 *   Modified by: Rin. Taniguchi    email: rin@is.kyushu-u.ac.jp
 *                Laboratory for Image and Media Understanding
 *                Dept. of Intelligent Systems
 *                Kyushu University, Japan
 *
 *      referring to: AM-CLR developed by the following people
 *
 *   Werner Jaeger        email: e9026603@student.tuwien.ac.at
 *   Wilfried Kubinger    email: wk@flexaut.tuwien.ac.at
 *
 *   Institute of Flexible Automation
 *   University of Technology Vienna
 *   Austria, Europe
 *
 *   amrgbReg.h  v1.0  1998/01/05
 */

/* Change History:

   - 1999/05/03 13:20:45 matthias
     ajustments to version 0.5.0 (card8/16 -> u_char/short)

 */

#ifndef _AMRGBREG_H
#define _AMRGBREG_H

#if defined(linux) && defined(__KERNEL__) || defined(__CONFIG__)

#define RGB_IMAGE_WIDTH_DEFAULT 640
#define RGB_IMAGE_HEIGHT_DEFAULT 480

/* AM-RGB Hardware Reference Manual; Imaging Technology Inc.*/
/* Document Number 47-H44007-00; July 1997 */

/* R-O   Read-Only Register */
/* W-O   Write-Only Register */
/* R/W   Read/Write Register */
/* R/W1C Clear on Write One to Register */
/* R/W0C Clear on Write Zero to Register */

/* AM-RGB Register Set (16bit access only), pages 5-1 to 5-62 */
/* ********************************************************** */


/* 
 * Prototypes
 */

/* read handler */


/* write handler */

int rgb_set_fpga (u_char, struct fpga_buffer_t *);

/* init handler */

void rgb_init_regs (u_char);
void amrgb_default_setting(struct amrgb_cam_t *);

/* misc handler */


/* end of prototypes */


/*
 * AM-RGB Register Definitions 
 */

#define RGB_MODID_OFFSET        0x00

#define RGB_REV_MASK            0x07
#define RGB_REV_SHIFT           0

#define RGB_TYPE_MASK		0xF8
#define RGB_TYPE_SHIFT		3
#define RGB_ID			0x8

#define RGB_MXCTL_OFFSET	0x01

#define RGB_MUX_RED		0
#define RGB_MUX_GREEN		1
#define RGB_MUX_BLUE		2

#define RGB_MUXSEL0_MASK	0x3
#define RGB_MUXSEL0_SHIFT	0
#define RGB_MUXSEL0_DEFAULT	RGB_MUX_RED
#define RGB_MUXSEL1_MASK	0xc
#define RGB_MUXSEL1_SHIFT	2
#define RGB_MUXSEL1_DEFAULT	RGB_MUX_GREEN
#define RGB_MUXSEL2_MASK	0x30
#define RGB_MUXSEL2_SHIFT	4
#define RGB_MUXSEL2_DEFAULT	RGB_MUX_BLUE

#define RGB_SDMODE_MASK		0x40
#define RGB_SDMODE_SHIFT	6
#define RGB_SDMODE_STATIC	0
#define RGB_SDMODE_DYNAMIC	1
#define RGB_SDMODE_DEFAULT	RGB_SDMODE_STATIC

/* R-O */
#define RGB_CSCSTAT_MASK	0x80
#define RGB_CSCSTAT_SHIFT	7
#define RGB_CSCSTAT_OFF		0
#define RGB_CSCSTAT_ON		1

#define RGB_FPGA_OFFSET		0x06

#define RGB_FPGA_CLEAR		0x07

#define RGB_HTOTAL_OFFSET	0x20
#define RGB_HTOTAL_MASK		0x3ff
#define RGB_HTOTAL_DEFAULT	454

#define RGB_HESYNC_OFFSET	0x21
#define RGB_HESYNC_MASK		0xff
#define RGB_HESYNC_DEFAULT	67

#define RGB_HEEQ_OFFSET		0x22
#define RGB_HEEQ_MASK		0xff
#define RGB_HEEQ_DEFAULT	32

#define RGB_SEREND_OFFSET	0x23
#define RGB_SEREND_MASK		0x3ff
#define RGB_SEREND_DEFAULT	388

#define RGB_VTOTAL_OFFSET	0x24
#define RGB_VTOTAL_MASK		0x7ff
#define RGB_VTOTAL_DEFAULT	524

#define RGB_VSYNC_OFFSET	0x25
#define RGB_VSEND_MASK		0x1f
#define RGB_VSEND_SHIFT		0
#define RGB_VSEND_DEFAULT	5
#define RGB_HSYNCPOL_MASK	0x40
#define RGB_HSYNCPOL_SHIFT	6
#define RGB_VSYNCPOL_MASK	0x80
#define RGB_VSYNCPOL_SHIFT	7
#define RGB_PLLCEN_MASK		0x100
#define RGB_PLLCEN_SHIFT	8
#define RGB_PLLCEN_DEFAULT	0

#define RGB_VSGTRT_OFFSET	0x26
#define RGB_VSGTRT_MASK		0x7ff
#define RGB_VSGTRT_DEFAULT	518

#define RGB_VGEND_OFFSET	0x27
#define RGB_VGEND_MASK		0x7f
#define RGB_VGEND_DEFAULT	11

#define RGB_HOFF_OFFSET		0x28
#define RGB_HOFF_MASK		0x3ff
#define RGB_HOFF_DEFAULT	119

#define RGB_HACTIVE_OFFSET	0x29
#define RGB_HACTIVE_MASK	0xfff
#define RGB_HACTIVE_DEFAULT	757

#define RGB_VOFF_OFFSET		0x2a
#define RGB_VOFF_MASK		0x3ff
#define RGB_VOFF_DEFAULT	15

#define RGB_VACTIVE_OFFSET	0x2b
#define RGB_VACTIVE_MASK	0x3ff
#define RGB_VACTIVE_DEFAULT	255

#define RGB_STRBDLY_OFFSET	0x2c
#define RGB_STRBDLY_MASK	0x3f
#define RGB_STRBDLY_DEFAULT	0

#define RGB_FRESET_OFFSET	0x2d
#define RGB_FROFF_MASK		0x3f
#define RGB_FROFF_SHIFT		0
#define RGB_FROFF_DEFAULT	0
#define RGB_FRSTMD_MASK		0x400
#define RGB_FRSTMD_SHIFT	10
#define RGB_FRSTMD_DISABLE	0
#define RGB_FRSTMD_ENABLE	1
#define RGB_FRSTMD_DEFAULT	RGB_FRSTMD_DISABLE
#define RGB_FRSTPOL_MASK	0x800
#define RGB_FRSTPOL_SHIFT	11
#define RGB_FRSTPOL_AHIGH	0
#define RGB_FRSTPOL_ALOW	1
#define RGB_FRSTPOL_DEFAULT	RGB_FRSTPOL_AHIGH
#define RGB_FRSTSZ_MASK		0x1000
#define RGB_FRSTSZ_SHIFT	12
#define RGB_FRSTSZ_FROFF	0
#define RGB_FRSTSZ_1LINE	1
#define RGB_FRSTSZ_DEFAULT	RGB_FRSTSZ_FROFF

#define RGB_CON1_OFFSET		0x2e

#define RGB_FENPOL_MASK		0x1
#define RGB_FENPOL_SHIFT	0
#define RGB_FENPOL_FALLING	0
#define RGB_FENPOL_RISING	1
#define RGB_FENPOL_DEFAULT	RGB_FENPOL_FALLING

#define RGB_LENPOL_MASK		0x2
#define RGB_LENPOL_SHIFT	1
#define RGB_LENPOL_FALLING	0
#define RGB_LENPOL_RISING	1
#define RGB_LENPOL_DEFAULT	RGB_LENPOL_FALLING

#define RGB_FLDPOL_MASK		0x4
#define RGB_FLDPOL_SHIFT	2
#define RGB_FLDPOL_DEFAULT	0

#define RGB_FLDSEL_MASK		0x8
#define RGB_FLDSEL_SHIFT	3
#define RGB_FLDSEL_DEFAULT	0

#define RGB_VSCAN_MASK		0x10
#define RGB_VSCAN_SHIFT		4
#define RGB_VSCAN_DISABLE	0
#define RGB_VSCAN_ENABLE	1
#define RGB_VSCAN_DEFAULT	RGB_VSCAN_DISABLE

#define RGB_TIMEMD_MASK		0x60
#define RGB_TIMEMD_SHIFT	5
#define RGB_TIMEMD_STRIPPED	0
#define RGB_TIMEMD_SEPARATE	2
#define RGB_TIMEMD_XTAL		3
#define RGB_TIMEMD_DEFAULT	RGB_TIMEMD_STRIPPED

#define RGB_VSYNCEN_MASK	0x80
#define RGB_VSYNCEN_SHIFT	7
#define RGB_VSYNCEN_DISABLE	0
#define RGB_VSYNCEN_ENABLE	1
#define RGB_VSYNCEN_DEFAULT	RGB_VSYNCEN_DISABLE

#define RGB_SYNCSEL_MASK	0x100
#define RGB_SYNCSEL_SHIFT	8
#define RGB_SYNCSEL_PTGHSYNC	0
#define RGB_SYNCSEL_PTGCCYNC	1
#define RGB_SYNCSEL_DEFAULT	RGB_SYNCSEL_PTGHSYNC

#define RGB_FLDSHFT_MASK	0x200
#define RGB_FLDSHFT_SHIFT	9
#define RGB_FLDSHFT_DISABLE	0
#define RGB_FLDSHFT_ENABLE	1
#define RGB_FLDSHFT_DEFAULT	RGB_FLDSHFT_DISABLE

#define RGB_VBSTAT_MASK		0x400 /* R-O */
#define RGB_VBSTAT_SHIFT	10
#define RGB_VBSTAT_BLANK	0
#define RGB_VBSTAT_ACTIVE	1

#define RGB_FLDSTAT_MASK	0x800 /* R-O */
#define RGB_FLDSTAT_SHIFT	11
#define RGB_FLDSTAT_EVEN	0
#define RGB_FLDSTAT_ODD		1

#define RGB_RGBTRIG_OFFSET	0x2f
#define RGB_TRIGEN_DISABLE	0
#define RGB_TRIGEN_EABLE	1
#define RGB_TRIGEN0_MASK	0x1
#define RGB_TRIGEN0_SHIFT	0
#define RGB_TRIGEN0_DEFAULT	RGB_TRIGEN_DISABLE

#define RGB_TRIGEN1_MASK	0x2
#define RGB_TRIGEN1_SHIFT	1
#define RGB_TRIGEN1_DEFAULT	RGB_TRIGEN_DISABLE

#define RGB_TRIGEN2_MASK	0x4
#define RGB_TRIGEN2_SHIFT	2
#define RGB_TRIGEN2_DEFAULT	RGB_TRIGEN_DISABLE

#define RGB_TRGCYC0_MASK	0x8 /* R-O */
#define RGB_TRGCYC0_SHIFT	3
#define RGB_TRGCYC1_MASK	0x10 /* R-O */
#define RGB_TRGCYC1_SHIFT	4
#define RGB_TRGCYC2_MASK	0x20 /* R-O */
#define RGB_TRGCYC2_SHIFT	5

#define RGB_TRIGINT_NOSET	0
#define RGB_TRIGINT_SET		1
#define RGB_TRIGINT0_MASK	0x40
#define RGB_TRIGINT0_SHIFT	6
#define RGB_TRIGINT1_MASK	0x80
#define RGB_TRIGINT1_SHIFT	7
#define RGB_TRIGINT2_MASK	0x100
#define RGB_TRIGINT2_SHIFT	8

#define RGB_TRGSRC_MASK		0x200
#define RGB_TRGSRC_SHIFT	9
#define RGB_TRGSRC_EXT		0
#define RGB_TRGSRC_SOFT		1
#define RGB_TRGSRC_DEFAULT	RGB_TRGSRC_EXT

#define RGB_TRGPOL_MASK		0x400
#define RGB_TRGPOL_SHIFT	10
#define RGB_TRGPOL_LOW		0
#define RGB_TRGPOL_HIGH		1
#define RGB_TRGPOL_DEFAULT	RGB_TRGPOL_LOW

#define RGB_TRIGLACE_MASK	0x800
#define RGB_TRIGLACE_SHIFT	11
#define RGB_TRIGLACE_INT	0
#define RGB_TRIGLACE_NOINT	1
#define RGB_TRIGLACE_DEFAULT	RGB_TRIGLACE_INT

#define RGB_STRBMD_MASK		0x3000
#define RGB_STRBMD_SHIFT	12
#define RGB_STRBMD_FASTNORM	0
#define RGB_STRBMD_SLOWNORM	1
#define RGB_STRBMD_FASTTRIG	2
#define RGB_STRBMD_SLOWTRIG	3
#define RGB_STRBMD_DEFAULT	RGB_STRBMD_FASTNORM

#define RGB_STRBPOL_MASK	0x4000
#define RGB_STRBPOL_SHIFT	14
#define RGB_STRBPOL_HIGH	0
#define RGB_STRBPOL_LOW		1
#define RGB_STRBPOL_DEFAULT	RGB_STRBPOL_HIGH

#define RGB_SKPFLDMD_MASK	0x8000
#define RGB_SKPFLDMD_SHIFT	15
#define RGB_SKPFLDMD_DISABLE	0
#define RGB_SKPFLDMD_ENABLE	1
#define RGB_SKPFLDMD_DEFAULT	RGB_SKPFLDMD_DISABLE

#define RGB_SWTRIG_NOSET	0
#define RGB_SWTRIG_SET		1
#define RGB_RGBSWTRIG_OFFSET	0x30
#define RGB_SWTRIG0_MASK	0x1
#define RGB_SWTRIG0_SHIFT	0
#define RGB_SWTRIG0_DEFAULT	RGB_SWTRIG_NOSET
#define RGB_SWTRIG1_MASK	0x2
#define RGB_SWTRIG1_SHIFT	1
#define RGB_SWTRIG1_DEFAULT	RGB_SWTRIG_NOSET
#define RGB_SWTRIG2_MASK	0x4
#define RGB_SWTRIG2_SHIFT	2
#define RGB_SWTRIG2_DEFAULT	RGB_SWTRIG_NOSET

#define RGB_PCLAMP_OFFSET	0x31
#define RGB_CCNT_MASK		0xf
#define RGB_CCNT_SHIFT		0
#define RGB_CCNT_DEFAULT	0

#define RGB_PCLMPOL_MASK	0x10
#define RGB_PCLMPOL_SHIFT	4
#define RGB_PCLMPOL_RISING	1
#define RGB_PCLMPOL_FALLING	0
#define RGB_PCLMPOL_DEFAULT	RGB_PCLMPOL_FALLING

#define RGB_CLMPSRC_MASK	0x20
#define RGB_CLMPSRC_SHIFT	5
#define RGB_CLMPSRC_STRIP	0
#define RGB_CLMPSRC_PROGRAM	1
#define RGB_CLMPSRC_DEFAULT	RGB_CLMPSRC_STRIP

#define RGB_LUTADDR_OFFSET	0x32
#define RGB_LUTADDR_MASK	0xffff

#define RGB_LUT0DATA_OFFSET	0x33
#define RGB_LUT0DATA_MASK	0xffff

#define RGB_LUT1DATA_OFFSET	0x34
#define RGB_LUT1DATA_MASK	0xffff

#define RGB_LUTDATA_OFFSET	0x35
#define RGB_LUTDATA_MASK	0xffff

#define RGB_MISCCTL_OFFSET	0x36
#define RGB_AMSMODE_MASK	0x1
#define RGB_AMSMODE_SHIFT	0
#define RGB_AMSMODE_NOINT	0
#define RGB_AMSMODE_INT		1
#define RGB_AMSMODE_DEFAULT	RGB_AMSMODE_INT

#define RGB_VCLKPOL_MASK	0x2
#define RGB_VCLKPOL_SHIFT	1
#define RGB_VCLKPOL_NOINV	0
#define RGB_VCLKPOL_INV	1
#define RGB_VCLKPOL_DEFAULT	RGB_VCLKPOL_NOINV

#define RGB_PLLCS_MASK		0x4
#define RGB_PLLCS_SHIFT		2
#define RGB_PLLCS_PROGRAM	0
#define RGB_PLLCS_NORM		1
#define RGB_PLLCS_DEFAULT	RGB_PLLCS_NORM

#define RGB_DACCS_MASK		0x8
#define RGB_DACCS_SHIFT		3
#define RGB_DACCS_PROGRAM	0
#define RGB_DACCS_NORM		1
#define RGB_DACCS_DEFAULT	RGB_DACCS_NORM

#define RGB_DACRDBK_MASK	0x10
#define RGB_DACRDBK_SHIFT	4
#define RGB_DACRDBK_DEFAULT	0

#define RGB_LPFSEL_MASK		0x40
#define RGB_LPFSEL_SHIFT	6
#define RGB_LPFSEL_ENABLE	0
#define RGB_LPFSEL_BYPASSED	1
#define RGB_LPFSEL_DEFAULT	RGB_LPFSEL_ENABLE

#define RGB_LUTOSEL_MASK	0x80
#define RGB_LUTOSEL_SHIFT	7
#define RGB_LUTOSEL_LUT0	0
#define RGB_LUTOSEL_LUT1	1
#define RGB_LUTOSEL_DEFAULT	RGB_LUTOSEL_LUT0

#define RGB_VIDINSEL_MASK	0x100
#define RGB_VIDINSEL_SHIFT	8
#define RGB_VIDINSEL_CAM0	0
#define RGB_VIDINSEL_CAM1	1
#define RGB_VIDINSEL_DEFAULT	RGB_VIDINSEL_CAM0

#define RGB_PATHSEL_MASK	0x200
#define RGB_PATHSEL_SHIFT	9
#define RGB_PATHSEL_SELECTED	0
#define RGB_PATHSEL_BYPASSED	1
#define RGB_PATHSEL_DEFAULT	RGB_PATHSEL_BYPASSED

#define RGB_INTSEL_MASK		0x400
#define RGB_INTSEL_SHIFT	10
/* #define RGB_INTSEL_DEFAULT	1*/ /* Interrupt by Vertical Blank */
#define RGB_INTSEL_DEFAULT 0
#define RGB_STRPSRC_MASK	0x800
#define RGB_STRPSRC_SHIFT	11
#define RGB_STRPSRC_HDRIVE	0
#define RGB_STRPSRC_GREEN	1
#define RGB_STRPSRC_DEFAULT	RGB_STRPSRC_HDRIVE

#define RGB_CSCTMOUT_MASK	0x1000
#define RGB_CSCTMOUT_SHIFT	12
#define RGB_WRINC_MASK		0x2000
#define RGB_WRINC_SHIFT		13
#define RGB_RDINC_MASK		0x4000
#define RGB_RDINC_SHIFT		14

#define RGB_DACPRG_OFFSET	0x37
#define RGB_DACSDATA_MASK	0x1
#define RGB_DACSDATA_SHIFT	0

#define RGB_PLLRPG_OFFSET	0x38
#define RGB_PLLSDATA_MASK	0x1
#define RGB_PLLSDATA_SHIFT	0

#define RGB_GAIN_28		0
#define RGB_GAIN_34		1
#define RGB_GAIN_25		2
#define RGB_GAIN_30		3
#define RGB_RGBGAIN_OFFSET	0x39
#define RGB_REDG_MASK		0x3
#define RGB_REDG_SHIFT		0
#define RGB_REDG_DEFAULT	RGB_GAIN_28
#define RGB_GRNG_MASK		0xc
#define RGB_GRNG_SHIFT		2
#define RGB_GRNG_DEFAULT	RGB_GAIN_28
#define RGB_BLUG_MASK		0x30
#define RGB_BLUG_SHIFT		4
#define RGB_BLUG_DEFAULT	RGB_GAIN_28

#define RGB_LSTAT_OFFSET	0x3a /* R-O */
#define RGB_LSTAT_MASK		0x3ff
#define RGB_LSTAT_SHIFT		0

#define RGB_INTCOEF_OFFSET	0x3c
/* Not yet implemented */

#define RGB_KDATAX_OFFSET	0x3d
/* Not yet implemented */

#define RGB_KDATAY_OFFSET	0x3e
/* Not yet implemented */

#define RGB_KDATAZ_OFFSET	0x3f
/* Not yet implemented */

/* DAC */
#define RGB_RPREF_DEFAULT	133
#define RGB_GPREF_DEFAULT	133
#define RGB_BPREF_DEFAULT	133
#define RGB_RNREF_DEFAULT	31
#define RGB_GNREF_DEFAULT	31
#define RGB_BNREF_DEFAULT	31
#define RGB_CLAMP_VOLT		31

/* PLL */

#define PLL_MASK		0x7ff

#define PLL_REG0_OFFSET		0
#define PLL_FDIV_MASK		0x7ff
#define PLL_FDIV_DEFAULT	779

#define PLL_REG1_OFFSET		1
#define PLL_LO_MASK		0xff
#define PLL_LO_DEFAULT		0

#define PLL_REG2_OFFSET		2
#define PLL_HI_MASK		0xff
#define PLL_HI_DEFAULT		0

#define PLL_REG3_OFFSET		3
#define PLL_RDIV_MASK		0x3ff
#define PLL_RDIV_SHIFT		0
#define PLL_RDIV_DEFAULT	0
#define PLL_REFPOL_MASK		0x400
#define PLL_REFPOL_SHIFT	10
#define PLL_REFPOL_DEFAULT	1

#define PLL_REG4_OFFSET		4
#define PLL_VCO_MASK		0x7
#define PLL_VCO_SHIFT		0
#define PLL_VCO_DEFAULT		5
#define PLL_PFD_MASK		0x38
#define PLL_PFD_SHIFT		3
#define PLL_PFD_DEFAULT		3
#define PLL_PDEN_MASK		0x40
#define PLL_PDEN_SHIFT		6
#define PLL_PDEN_DEFAULT	1
#define PLL_INTFLT_MASK		0x80
#define PLL_INTFLT_SHIFT	7
#define PLL_INTFLT_DEFAULT	0
#define PLL_INTVCO_MASK		0x100
#define PLL_INTVCO_SHIFT	8
#define PLL_INTVCO_DEFAULT	1
#define PLL_CLKSEL_MASK		0x200
#define PLL_CLKSEL_SHIFT	9
#define PLL_CLKSEL_DEFAULT	1
#define PLL_AUX4_SHIFT		10
#define PLL_AUX4_DEFAULT	1

#define PLL_REG5_OFFSET		5
#define PLL_FBKSEL_MASK		0x1
#define PLL_FBKSEL_SHIFT	0
#define PLL_FBKSEL_DEFAULT	1 /* Internal Feedback */
#define PLL_FBKPPOL_MASK	0x2
#define PLL_FBKPOL_SHIFT	1
#define PLL_FBKPOL_DEFAULT	1 /* Internal Feedback */
#define PLL_PDA_MASK		0x30
#define PLL_PDA_SHIFT		4
#define PLL_PDA_DEFAULT		2
#define PLL_PDB_MASK		0xc0
#define PLL_PDB_SHIFT		6
#define PLL_PDB_DEFAULT		3
#define PLL_LDLG_MASK		0x100
#define PLL_LDLG_SHIFT		8
#define PLL_LDLG_DEFAULT	1
#define PLL_FINEEN_MASK		0x200
#define PLL_FINEEN_SHIFT	9
#define PLL_FINEEN_DEFAULT	0
#define PLL_AUX5_SHIFT		10
#define PLL_AUX5_DEFAULT	1

#define PLL_REG6_OFFSET		6
#define PLL_LCNT_MASK		0x7
#define PLL_LCNT_SHIFT		0
#define PLL_LCNT_DEFAULT	5
#define PLL_OMUX1_MASK		0x8
#define PLL_OMUX1_SHIFT		3
#define PLL_OMUX1_DEFAULT	0
#define PLL_EXTREF_MASK		0x400
#define PLL_EXTREF_SHIFT	10
#define PLL_EXTREF_DEFAULT	1


#endif /* __KERNEL__ || __CONFIG__ */

#endif /* _AMRGBREG_H */

