/*   video test program for the AM-STD module  
 *   free public release
 *
 *   test_icpci_cmp.c  v1.0  1999/05/05
 */

/* Change History:

   1999/09/21 16:47:23  matthias
   ajustments for version 0.5.1
   the statistics are now u_long values
   due to the capability of decimazation we have to distinguish between
   MEMWIDTH/HEIGHT and BMAWIDTH/HEIGHT
   RAWSIZE is now MEMBUFSIZE ans PAGEDSIZE is now BMABUFSIZE
   MEMSIZE is renamed to MEMMAPSIZE

 */

/*
 * Defines:
 */

/*
 * Includes:
 */

#include <stdlib.h>
#include <unistd.h>
#include <getopt.h>
#include <string.h>
#include <signal.h>
#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <sys/mman.h>

#include <asm/page.h>

#include "icpciExt.h"

#define  EXOFILE "amstd.exo"
#define  FRAMES 1

typedef enum {
  READ_MODE,
  MMAP_MODE,
  BMA_MODE
} operation_mode_t;

typedef enum {
  GRAB_MODE,
  SNAP_MODE,
  FREEZE_MODE
} acquisition_mode_t;

/*
 * Devices:
 */

const char *cmp_camera_device[] = {
  "/dev/ic0cmp0" , "/dev/ic0cmp1" , 
  "/dev/ic1cmp0" , "/dev/ic1cmp1" ,
  "/dev/ic2cmp0" , "/dev/ic2cmp1" ,
  "/dev/ic3cmp0" , "/dev/ic3cmp1"
};

const char *cmp_ioctl_device[] = {
  "/dev/ic0cmpI",
  "/dev/ic1cmpI",
  "/dev/ic2cmpI",
  "/dev/ic3cmpI"
};

const char *cmp_ident_device[] = {
  "/dev/ic0idI",
  "/dev/ic1idI",
  "/dev/ic2idI",
  "/dev/ic3idI" 
};

/*
 * Globals:
 */

void	       *image_mmaped = NULL;	/* pointer to mmapped frame buffer
					   (on-board or upper memory) */

size_t		mem_size = 0UL;		/* size of on-board/uppen memory */


int framegrabber=0;
operation_mode_t        operation_mode = BMA_MODE;
acquisition_mode_t      acquisition_mode = GRAB_MODE;


/*
 * Prototypes:
 */

void
Clean_Up (void) {

  sleep (1);
  
  if (image_mmaped != NULL && mem_size != 0) {
    munmap (image_mmaped, mem_size);
  }

  if (framegrabber != -1) {
    close (framegrabber);
  }
  
  exit (-1);
}

void write_image_to_file
 (int frame_count, u_int *read_buffer, int width, int height) {

  char str[64];
  int j, k, o;
  u_char *buf;

  sprintf(str, "im.%03d.ppm", frame_count);
  fprintf(stderr, "Writing image %d\n", frame_count);
  if ((o = open(str, O_WRONLY | O_CREAT, 0644)) < 0) {
    perror("ppm open failed");
    Clean_Up();
  }
  
  /* make PPM header and save to file */
  sprintf(str, "P6\n%d %d\n255\n", 640, 480);
  write (o, &str[0], strlen(str));
  /* save the data to PPM file */

  buf = (u_char *)read_buffer;
  for (j=0; j<480; j++) {
    for (k=0; k<640; k++) {
      write(o, buf, 3);
      buf+=4;
    }
  }
  
  close(o);
}

void write_interlace_image_to_file
 (int frame_count, u_int *read_buffer, int width, int height) {

  char str[64];
  int j, k, o;
  u_char *buf0, *buf1;

  sprintf(str, "im.%03d.ppm", frame_count);
  fprintf(stderr, "Writing image %d\n", frame_count);
  if ((o = open(str, O_WRONLY | O_CREAT, 0644)) < 0) {
    perror("ppm open failed");
    Clean_Up();
  }
  
  /* make PPM header and save to file */
  sprintf(str, "P6\n%d %d\n255\n", 640, 480);
  write (o, &str[0], strlen(str));
  /* save the data to PPM file */

  buf0 = (u_char *)read_buffer;
  buf1 = (u_char *)read_buffer + 640 * 240 *4;
  for (j=0; j<240; j++) {
    for (k=0; k<640; k++) {
      write(o, buf0, 3);
      buf0+=4;
    }
    for (k=0; k<640; k++) {
      write(o, buf1, 3);
      buf1+=4;
    }
  }
  
  close(o);
}


/*
 * Main:
 */

void main(int argc, char **argv) {


  fd_set wait_fdset;
  struct timeval wait_time;
  int frame_count;

  struct icp_info_t *icp_info = NULL;

  int board_arg = 0, camera_arg = 0;

  int width=0, height=0, rawsize=0, pagedsize=0;
  u_int *read_buffer=NULL;

  static struct sigaction ignore_action = { SIG_IGN,
					    0,
					    0,
					    NULL };

  switch (operation_mode) {
  case READ_MODE:
    if ((framegrabber = open (cmp_camera_device[board_arg * 4 + camera_arg],
                              O_RDONLY)) < 0) {
      perror ("open framegrabber device (with AM-CLR)");
      exit (-1);
    }
    break;

  case MMAP_MODE:
    if ((framegrabber = open (cmp_ioctl_device[board_arg],
			    O_RDONLY)) < 0) {
      perror ("open camera device");
      exit (-1);
    }
    break;

  case BMA_MODE:
    if ((framegrabber = open (cmp_ident_device[board_arg],
			    O_RDONLY)) < 0) {
      perror ("open camera device");
      exit (-1);
    }
  }

  if (operation_mode != READ_MODE) {
    /* MMAP_MODE or BMA_MODE */
    if (ioctl (framegrabber, GIOC_GE_SET_CAMERA, &camera_arg) < 0) {
      perror ("GIOC_GE_SET_CAMERA");
      Clean_Up();
    }
    if (sigaction (SIGIO, &ignore_action, NULL) < 0) {
      perror ("Can't set handler to ignore action");
      Clean_Up ();
    }
    if (fcntl (framegrabber, F_SETOWN, getpid()) < 0) {
      perror ("F_SETOWN");
      Clean_Up ();
    }
    if (fcntl (framegrabber, F_SETFL,
	       fcntl (framegrabber, F_GETFL, NULL) | FASYNC) < 0) {
      perror ("F_SETFL FASYNC");
      Clean_Up ();
    }
  }

  /* --- get image dimensions --------------------------------------------- */

  switch (operation_mode) {
  case READ_MODE:
  case MMAP_MODE:
    if (ioctl (framegrabber, GIOC_GE_GET_MEMWIDTH, &width) < 0) {
      perror ("GIOC_GE_GET_WIDTH");
      Clean_Up();
    }
    if (ioctl (framegrabber, GIOC_GE_GET_MEMHEIGHT, &height) < 0) {
      perror ("GIOC_GE_GET_HEIGHT");
      Clean_Up();
    }
    break;
    
  case BMA_MODE:
    if (ioctl (framegrabber, GIOC_GE_GET_BMAWIDTH, &width) < 0) {
      perror ("GIOC_GE_GET_WIDTH");
      Clean_Up();
    }
    if (ioctl (framegrabber, GIOC_GE_GET_BMAHEIGHT, &height) < 0) {
      perror ("GIOC_GE_GET_HEIGHT");
      Clean_Up();
    }
    break;
  }

  if (ioctl (framegrabber, GIOC_GE_GET_MEMBUFSIZE, &rawsize) < 0) {
    perror ("GIOC_GE_GET_RAWSIZE");
    Clean_Up();
  }
  if (ioctl (framegrabber, GIOC_GE_GET_BMABUFSIZE, &pagedsize) < 0) {
    perror ("GIOC_GE_GET_PAGEDSIZE");
    Clean_Up();
  }

  /* --- initialize the given operation mode ------------------------------ */

  switch (operation_mode) {
  case READ_MODE:
    if ((read_buffer = (u_int *) malloc (rawsize)) == NULL) {
      perror ("can't malloc read buffer.\n");
      Clean_Up();
    }
    break;
  case MMAP_MODE:
    if (ioctl (framegrabber, GIOC_GE_GET_MEMMAPSIZE, &mem_size) < 0) {
      perror ("GIOC_GE_GET_MEMSIZE");
      exit (-1);
    }
    if ((image_mmaped = (u_int*)mmap (0, mem_size, PROT_READ, MAP_SHARED,
				       framegrabber , 0)) == (void*)-1) {
      perror ("Can't map camera device");
      exit (-1);
    }
    break;
    
  case BMA_MODE:
    if (ioctl (framegrabber, GIOC_GE_GET_BMAMAPSIZE, &mem_size) < 0) {
      perror ("GIOC_GE_GET_BMASIZE");
      exit (-1);
    }
    if ((image_mmaped = (void*)mmap (0, mem_size, PROT_READ, MAP_SHARED,
				     framegrabber, 0)) == (void*)-1) {
      perror ("Can't map camera device");
      exit (-1);
    }
    /* pointer to the info structure of the acquired image */
    icp_info = (struct icp_info_t *)(((u_char *)image_mmaped) + rawsize);
    break;
  }

  if (operation_mode != READ_MODE) {
    /* MMAP_MODE or BMA_MODE */
    if (ioctl (framegrabber, GIOC_CR_SET_GRAB, NULL) < 0) {
      perror ("GIOC_CR_SET_GRAB");
      Clean_Up();
    }
  }

  frame_count = 0;
  while (frame_count < FRAMES) {

    switch (acquisition_mode) {

    case GRAB_MODE:
      switch(operation_mode) {
   
      case READ_MODE:
        if (read (framegrabber, read_buffer, rawsize) < 0) {
          if (errno == ENODATA) {
	    fprintf(stderr, "*** No Sync ***");
	  }
          else {
            perror ("read framegrabber device");
	    Clean_Up();
          }        
        }

        write_image_to_file (frame_count++, read_buffer, width, height);
        break;

      case MMAP_MODE:
        FD_ZERO (&wait_fdset);
        FD_SET (framegrabber, &wait_fdset);	    
        wait_time.tv_sec = 1;
        wait_time.tv_usec = 0;
        switch (select (getdtablesize(), &wait_fdset, NULL, NULL, 
                        &wait_time)) {
        case -1:
	  fprintf(stderr, "*** Signal in select ***\n");
          break;
        case 0:
          fprintf(stderr, "*** No Sync ***\n");
          break;
        case 1:
	  write_interlace_image_to_file (frame_count++, image_mmaped,
					 width, height);
	  break;
        default:
          perror ("select got unexpected return value");
	  Clean_Up();
        }
        break;

      case BMA_MODE:
        FD_ZERO (&wait_fdset);
        FD_SET (framegrabber, &wait_fdset);	    
        wait_time.tv_sec = 1;
        wait_time.tv_usec = 0;
        switch (select (getdtablesize(), &wait_fdset, NULL, NULL, 
                        &wait_time)) {
        case -1:
	  fprintf(stderr,"*** Signal in select ***\n");
          break;
        case 0:
          fprintf(stderr, "*** No Sync ***\n");
          break;
        case 1:
	  write_image_to_file (frame_count++, image_mmaped, width, height);
	  break;
        default:
	  perror ("select got unexpected return value");
	  Clean_Up();
        }
        break;
  
      default:
        perror("unknown acquisition mode.\n");
	Clean_Up();
      }
      break;
       
 
    default:
      perror("unknown operation mode.\n");
      Clean_Up();
    }

  }

  if (operation_mode != READ_MODE) {
    /* MMAP_MODE or BMA_MODE */
    if (ioctl (framegrabber, GIOC_CR_SET_FREEZE, NULL) < 0) {
      perror ("GIOC_CR_SET_FREEZE");
      Clean_Up();
    }
    if (operation_mode == BMA_MODE) {
      /* output info of last acquired image */
      printf ("clock tics: %ld\n", icp_info->clock_tics);
      printf ("time stamp: %d sec, %d usec\n", 
	      (int)icp_info->time_stamp.tv_sec,
	      (int)icp_info->time_stamp.tv_usec);
      printf ("frames captured: %ld\n", icp_info->frame_nums.frames_captured);
      printf ("   frames valid: %ld\n", icp_info->frame_nums.frames_valid);
      printf ("    frames lost: %ld\n", icp_info->frame_nums.frames_lost);
    }
  }

  if (read_buffer != NULL) free (read_buffer);

  if (operation_mode != READ_MODE) {
    if (munmap (image_mmaped, mem_size) < 0) {
      perror ("unmap framebuffer");
      exit (-1);
    }
  } 

  if (close (framegrabber) < 0) {
    perror ("close framegrabber device");
    exit (-1);
  }
}
 

